<?php

/**
 * SecureTrading STPP Shopping Carts
 * Abantecart 0.9.1
 * Module Version 2.5.7
 * Last Updated 13 March 2013
 * Written by Peter Barrow for SecureTrading Ltd.
 * http://www.securetrading.com
 */

?><?php
 
class AbanteSTAPI extends AbstractSTAPI
{
	/**
	 * Handles errors differently depending on whether we come from the AJAX checkout or elsewhere (inc. the 3D Callback) in the module
	 * 
	 * If we are in the checkout, we must save an error message to the session and return.
	 * If we are not in the checkout we must save the error message to the session, redirect to checkout/payment and exit.
	 */
	protected function handleError($message, $params) {
		
		if ($params['checkout']) {
			$_SESSION['securetrading_err_message'] = $message;
			return;
		}
		
		$_SESSION['error'] = $message;
		header("Location: " . HTTP_SERVER . INDEX_FILE . '?rt=checkout/payment');
		exit;
	}
	
	/**
	 * Save 3D Secure information into session.
	 */
	protected function before3dRedirectAction($params) {
	
	// Save 3D Secure information into session.  Will be used after performing 3DQ when posting to ACS URL.
		$_SESSION['securetrading_3ds_info'] = array(
			'acsurl' => $params['result']->acsurl,
			'md' => $params['result']->md,
			'pareq' => $params['result']->pareq,
			'termurl' => $params['termurl'],
		);
		
		// Return FALSE to inform the framework to not try and redirect the client automatically.  This will, in effect, eventually cause runSTAPI() to fail.
		return FALSE;
	}
	
	/**
	 * Process a transaction.
	 */
	protected function processPayment($params) {
	
		// Create the AVS string:
		$avsString = $this->createAvsString($params);
		
		// Reset the order status:
		$params['st_model']->resetOrderStatus($params['result']->orderreference);
		
		// Update the order:
		$params['model']->_confirm($params['result']->orderreference, $params['successful_order_status_id'], $avsString);
		
		// Return TRUE;
		return TRUE;
	}
	
	/**
	 * Updates the status of failed orders.
	 * 
	 * This method will work differently depending on whether we come from the AJAX checkout or a 3D callback.
	 * 3D callbacks do not use AJAX and will perform a header redirect back to checkout/confirm.  This creates a new order automatically (in core code).
	 * The AJAX checkout will not reload checkout/confirm if the transaction fails so we will have to create a new order manually.
	 * This is so that both 3DS callbacks and non 3DS transactions update their orders in the same way.
	 */
	protected function processPaymentFailure($params) {
		
		// Create the AVS string:
		$avsString = $this->createAvsString($params);
		
		// Select the order status:
		$orderStatus = $params['result']->errorcode == 70000 ? (int) $params['declined_order_status_id'] : (int) $params['failed_order_status_id'];
		
		// If we are coming from the AJAX checkout (and NOT from a 3D AVS redirect):
		if ($params['checkout']) { // If we are coming from the AJAX checkout (and NOT from a 3D Callback)
		
			// Retrieve the order data:
			$orderData = $params['model']->getOrder($params['result']->orderreference);
			
			// Update the order status:
			$params['st_model']->updateFailedOrder($params['result']->orderreference, $orderStatus);
			
			// Create a 'new' order so that the customer can re-enter their card details:
			$orderId = $params['model']->create($orderData);
			
			// Put this new order ID into the session.  Required:
			$_SESSION['order_id'] = $orderId;
			
			// Return FALSE:
			return FALSE;
		}
		
		// If we are coming from a 3D callback:
			$params['st_model']->updateFailedOrder($params['result']->orderreference, $orderStatus);
		
		// Return FALSE:
		return FALSE;
	}
	
	protected function process3dSecure($params) {
		return $this->processPayment($params);
	}
	
	protected function process3dSecureFailure($params) {
		return $this->processPaymentFailure($params);
	}
	
	/**
	 * Facade method for encapsulating the AVS string.
	 */
	private function createAvsString($params) {
	
		// Prepare AVS information:
		$securityAddress = $params['result']->securityaddress ? $params['result']->securityaddress : 'Not Matched';
		$securityPostcode = $params['result']->securitypostcode ? $params['result']->securitypostcode : 'Not Matched';
		$securityCode = $params['result']->securitycode ? $params['result']->securitycode : 'Not Matched';
		
		// Create AVS String:
		return "
			AVS Address: {$securityAddress}\n
			AVS Postcode: {$securityPostcode}\n
			AVS Security Code: {$securityCode}
		";
	}
}

?>